import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

public class Main extends JFrame {

    private static final int DELAY = 1700;

    private class DemoPanel extends JPanel {

        // clock ticks every DELAY milliseconds

        private int s = 0;
        public void tick() { s = s+1; }

        // helper functions

        // returns the average of the inputs. Tip: good for finding midpoints
        double average(double v1, double v2) {
            return (v1+v2) / 2.0;
        }

        // returns the average of the inputs. Tip: average(x1,x1,x2) is good
        // for finding a point that is 1/3 of the way towards another point
        double average(double v1, double v2, double v3) {
            return (v1+v2+v3) / 3.0;
        }

        // returns the x coordinate of point (x,y) rotated angle degrees around
        // point (centerX,centerY); used in constructing triangles etc.
        double rotateX(double centerX, double centerY, double x, double y, double angle) {
            angle = Math.PI * 2.0 * (angle / 360.0);
            return centerX + (x-centerX) * Math.cos(angle) - (y-centerY) * Math.sin(angle);
        }

        // returns the y coordinate of point (x,y) rotated angle degrees around
        // point (centerX,centerY); used in constructing triangles etc.
        double rotateY(double centerX, double centerY, double x, double y, double angle) {
            angle = Math.PI * 2.0 * (angle / 360.0);
            return centerY + (x-centerX) * Math.sin(angle) + (y-centerY) * Math.cos(angle);
        }

        // example 1

        void spiral(double angle, int n, Graphics g) {
            if (getWidth() < n) { return; }
            int x = (int)rotateX(0,0,n,0,angle) + getWidth() / 2;
            int y = (int)rotateY(0,0,n,0,angle) + getHeight() / 2;
            g.fillOval(x-1,y-1,2,2); // this draws an oval
            spiral(angle + 0.2,n+1,g); // this is a recursive call that draws the rest of the spiral
        }

        // example 2

        void tree(double dx, double x, double y, Graphics g) {
            if (dx < 0.5) { return; }
            double new_dx = dx / 2;
            double new_y = y - 20;
            double new_x1 = x - new_dx;
            double new_x2 = x + new_dx;
            int c = (int)(256 * (x / getWidth()));
            g.setColor(new Color(255,0,c));
            g.drawLine((int)x,(int)y,(int)new_x1,(int)new_y);
            g.drawLine((int)x,(int)y,(int)new_x2,(int)new_y);
            tree(new_dx,new_x1,new_y,g); // this is a recursive call draws the left side
            tree(new_dx,new_x2,new_y,g); // this is a recursive call draws the right side
        }

        // exercise 1

        void boxes(double x, double y, double width, double height, Graphics g) {
            if (width < 2) { return; }
            g.drawRect((int)x,(int)y,(int)width,(int)height);
            boxes(x,y,width/2,width/2,g);
            boxes(x+width/2,y+height/2,width/2,width/2,g);
        }

        // exercise 2

        void tree(double x, double y, double step, double angle, Graphics g, double d) {
            if (step < 2) { return; }
            double x1 = rotateX(x,y,x,y-step,angle) + d;
            double y1 = rotateY(x,y,x,y-step,angle);
            g.drawLine((int)x,(int)y,(int)(x1),(int)(y1));
            tree(x1,y1,step * 0.7,angle+25,g,d);
            tree(x1,y1,step * 0.7,angle-25,g,d);
        }

        // exercise 3

        void recTriangle(double x1, double y1, double x2, double y2, double x3, double y3, int n, Graphics g) {
            if (n < 0) { return; }
            g.drawLine((int)x1,(int)y1,(int)x2,(int)y2);
            g.drawLine((int)x2,(int)y2,(int)x3,(int)y3);
            g.drawLine((int)x3,(int)y3,(int)x1,(int)y1);
            double x12 = average(x1,x2);
            double y12 = average(y1,y2);
            double x23 = average(x2,x3);
            double y23 = average(y2,y3);
            double x31 = average(x3,x1);
            double y31 = average(y3,y1);
            recTriangle(x12,y12,x23,y23,x2,y2,n-1,g);
            recTriangle(x23,y23,x31,y31,x3,y3,n-1,g);
            recTriangle(x31,y31,x12,y12,x1,y1,n-1,g);
        }

        // exercise 4

        void koch_line(double x1, double y1, double x2, double y2, int n, Graphics g) {
            if (n == 0) {
                g.drawLine((int)x1,(int)y1,(int)x2,(int)y2);
                return;
            }
            double x3 = average(x1,x1,x2);
            double y3 = average(y1,y1,y2);
            double x4 = average(x1,x2,x2);
            double y4 = average(y1,y2,y2);
            double x5 = rotateX(x3,y3,x4,y4,60);
            double y5 = rotateY(x3,y3,x4,y4,60);
            koch_line(x1,y1,x3,y3,n-1,g);
            koch_line(x3,y3,x5,y5,n-1,g);
            koch_line(x5,y5,x4,y4,n-1,g);
            koch_line(x4,y4,x2,y2,n-1,g);
        }

        void koch_triangle(double x1, double y1, double x2, double y2, double x3, double y3, int n, Graphics g) {
            koch_line(x1,y1,x2,y2,n,g);
            koch_line(x2,y2,x3,y3,n,g);
            koch_line(x3,y3,x1,y1,n,g);
        }

        // exercise 5

        void koch_triangle_alt(double x1, double y1, double x2, double y2, double x3, double y3, int n, Graphics g) {
            koch_line(x1,y1,x3,y3,n,g);
            koch_line(x3,y3,x2,y2,n,g);
            koch_line(x2,y2,x1,y1,n,g);
        }

        public void paintComponent(Graphics g) {
            super.paintComponent(g);
            g.setColor(Color.BLACK);
            g.fillRect(0,0,this.getWidth(),this.getHeight());
            g.setColor(Color.WHITE);

            // compute coordinates for a triangle
            double x = getWidth()/2;
            double y = getHeight() * 0.72;
            double x1 = x - (getHeight() * 0.35);
            double x2 = x + (getHeight() * 0.35);
            double y1 = y;
            double y2 = y;
            double x3 = rotateX(x1,y1,x2,y2,-60);
            double y3 = rotateY(x1,y1,x2,y2,-60);

            int exercise_index = 0;

            if (35 < s) { s = 0; }

            if (s < 3) {
                exercise_index = 1;
                g.setColor(Color.YELLOW);
                int w = (int)(getHeight() * 0.4);
                boxes(getWidth()/2 - w, getHeight()/2 - w, 2 * w, 2 * w, g);
            } else if (s < 6) {
                exercise_index = 2;
                g.setColor(Color.WHITE);
                tree(getWidth()/2,getHeight(),getHeight()/4,0,g,0);
            } else if (s < 12) {
                exercise_index = 3;
                g.setColor(Color.MAGENTA);
                recTriangle(x1,y1,x2,y2,x3,y3,s-6,g);
            } else if (s < 17) {
                exercise_index = 4;
                g.setColor(Color.GREEN);
                koch_triangle(x1,y1,x2,y2,x3,y3,s-12,g);
            } else if (s < 22){
                exercise_index = 5;
                g.setColor(Color.RED);
                koch_triangle_alt(x1,y1,x2,y2,x3,y3,s-17,g);
            } else {
                exercise_index = -1;
                int si = s % 4;
                int w = (int)(getHeight() * 0.2);
                int xi = getWidth()/2 - w + getWidth()/4;
                int yi = getHeight()/2;
                // the house
                g.setColor(Color.RED);
                boxes(xi, yi, 2 * w, 2 * w, g);
                g.setColor(Color.WHITE);
                recTriangle(xi-30,yi,xi+30 + 2 * w,yi,xi + w,getHeight()/5,si,g);
                // the tree
                g.setColor(Color.GREEN);
                tree(getWidth()/4,yi + 2 * w,getHeight()/6,0,g,1 * (si-2));
                // the sun
                x = getWidth() * 0.40;
                y = getHeight() * 0.30;
                x1 = x - (getHeight() * 0.1);
                x2 = x + (getHeight() * 0.1);
                y1 = y;
                y2 = y;
                x3 = rotateX(x1,y1,x2,y2,-60);
                y3 = rotateY(x1,y1,x2,y2,-60);
                g.setColor(Color.YELLOW);
                koch_triangle(x1,y1,x2,y2,x3,y3,1 + (s % 3),g);
            }

            if (0 <= exercise_index) {
                g.setColor(Color.WHITE);
                g.fillRect(10,10,82,30);
                g.setColor(Color.BLACK);
                g.drawString("Uppgift " + exercise_index,22,30);
                g.setColor(Color.WHITE);
            }
        }

    }
    public Main() {
        setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        setSize(500,400); setLocation(50,50);
        DemoPanel panel = new DemoPanel();
        add(panel); setVisible(true);
        ActionListener al = (e -> { panel.repaint(); panel.tick(); });
        Timer t = new Timer(DELAY,al); t.start();
    }
    public static void main(String[] args) {
        Main f = new Main();
    }
}
